@@ -140,7 +140,6 @@
 #include <rec_type.h>
 #include <off_cvt.h>
 #include <mark_corrupt.h>
-#include <quote_821_local.h>
 #include <quote_822_local.h>
 #include <mail_proto.h>
 #include <mime_state.h>
@@ -337,6 +336,8 @@ int     smtp_helo(SMTP_STATE *state)
 	&& (state->misc_flags & SMTP_MISC_FLAG_IN_STARTTLS) == 0) {
 	/* XXX Mix-up of per-session and per-request flags. */
 	state->misc_flags |= SMTP_MISC_FLAG_IN_STARTTLS;
+	smtp_stream_setup(state->session->stream, var_smtp_starttls_tmout,
+			  var_smtp_req_deadline, 0);
 	tls_helo_status = smtp_start_tls(state);
 	state->misc_flags &= ~SMTP_MISC_FLAG_IN_STARTTLS;
 	return (tls_helo_status);
@@ -347,7 +348,7 @@ int     smtp_helo(SMTP_STATE *state)
      * Prepare for disaster.
      */
     smtp_stream_setup(state->session->stream, var_smtp_helo_tmout,
-		      var_smtp_rec_deadline);
+		      var_smtp_req_deadline, 0);
     if ((except = vstream_setjmp(state->session->stream)) != 0)
 	return (smtp_stream_except(state, except, where));
 
@@ -763,7 +764,7 @@ int     smtp_helo(SMTP_STATE *state)
 	     * Prepare for disaster.
 	     */
 	    smtp_stream_setup(state->session->stream, var_smtp_starttls_tmout,
-			      var_smtp_rec_deadline);
+			      var_smtp_req_deadline, 0);
 	    if ((except = vstream_setjmp(state->session->stream)) != 0)
 		return (smtp_stream_except(state, except,
 					"receiving the STARTTLS response"));
@@ -904,7 +905,7 @@ static int smtp_start_tls(SMTP_STATE *state)
 		    | SMTP_KEY_FLAG_ADDR);
 
     if (state->tls->conn_reuse) {
-	TLS_PARAMS tls_params;
+	TLS_CLIENT_PARAMS tls_params;
 
 	/*
 	 * Send all our wishes in one big request.
@@ -970,7 +971,7 @@ static int smtp_start_tls(SMTP_STATE *state)
 			   session->stream, STR(iter->addr),
 			   STR(port_buf), var_smtp_starttls_tmout,
 			   var_smtp_data2_tmout, state->service,
-			   tls_proxy_params_from_config(&tls_params),
+			   tls_proxy_client_param_from_config(&tls_params),
 			   &init_props, &start_props);
 	vstring_free(port_buf);
 
@@ -1111,18 +1112,17 @@ static int smtp_start_tls(SMTP_STATE *state)
      * "QUIT".
      * 
      * See src/tls/tls_level.c and src/tls/tls.h. Levels above "encrypt" require
-     * matching.  Levels >= "dane" require CA or DNSSEC trust.
+     * matching.
      * 
-     * When DANE TLSA records specify an end-entity certificate, the trust and
-     * match bits always coincide, but it is fine to report the wrong
-     * end-entity certificate as untrusted rather than unmatched.
+     * NOTE: We use "IS_MATCHED" to satisfy policy, but "IS_SECURED" to log
+     * effective security.  Thus "half-dane" is never "Verified" only
+     * "Trusted", but matching is enforced here.
+     * 
+     * NOTE: When none of the TLSA records were usable, "dane" and "half-dane"
+     * fall back to "encrypt", updating the tls_context level accordingly, so
+     * we must check that here, and not state->tls->level.
      */
-    if (TLS_MUST_TRUST(state->tls->level))
-	if (!TLS_CERT_IS_TRUSTED(session->tls_context))
-	    return (smtp_site_fail(state, DSN_BY_LOCAL_MTA,
-				   SMTP_RESP_FAKE(&fake, "4.7.5"),
-				   "Server certificate not trusted"));
-    if (TLS_MUST_MATCH(state->tls->level))
+    if (TLS_MUST_MATCH(session->tls_context->level))
 	if (!TLS_CERT_IS_MATCHED(session->tls_context))
 	    return (smtp_site_fail(state, DSN_BY_LOCAL_MTA,
 				   SMTP_RESP_FAKE(&fake, "4.7.5"),
@@ -1178,7 +1178,8 @@ static void smtp_text_out(void *context, int rec_type,
 	if (state->space_left == var_smtp_line_limit
 	    && data_left > 0 && *data_start == '.')
 	    smtp_fputc('.', session->stream);
-	if (var_smtp_line_limit > 0 && data_left >= state->space_left) {
+	if (ENFORCING_SIZE_LIMIT(var_smtp_line_limit)
+	    && data_left >= state->space_left) {
 	    smtp_fputs(data_start, state->space_left, session->stream);
 	    data_start += state->space_left;
 	    data_left -= state->space_left;
@@ -1386,6 +1387,65 @@ static void smtp_mime_fail(SMTP_STATE *state, int mime_errs)
 		   "%s", detail->text);
 }
 
+/* smtp_out_raw_or_mime - output buffer, raw output or MIME-aware */
+
+static int smtp_out_raw_or_mime(SMTP_STATE *state, int rec_type, VSTRING *buf)
+{
+    SMTP_SESSION *session = state->session;
+    int     mime_errs;
+
+    if (session->mime_state == 0) {
+	smtp_text_out((void *) state, rec_type, vstring_str(buf),
+		      VSTRING_LEN(buf), (off_t) 0);
+    } else {
+	mime_errs =
+	    mime_state_update(session->mime_state, rec_type,
+			      vstring_str(buf), VSTRING_LEN(buf));
+	if (mime_errs) {
+	    smtp_mime_fail(state, mime_errs);
+	    return (-1);
+	}
+    }
+    return (0);
+}
+
+/* smtp_out_add_header - format address header, uses session->scratch* */
+
+static int smtp_out_add_header(SMTP_STATE *state, const char *label,
+			               const char *lt, const char *addr,
+			               const char *gt)
+{
+    SMTP_SESSION *session = state->session;
+
+    smtp_rewrite_generic_internal(session->scratch2, addr);
+    vstring_sprintf(session->scratch, "%s: %s", label, lt);
+    smtp_quote_822_address_flags(session->scratch,
+				 vstring_str(session->scratch2),
+				 QUOTE_FLAG_DEFAULT | QUOTE_FLAG_APPEND);
+    vstring_strcat(session->scratch, gt);
+    return (smtp_out_raw_or_mime(state, REC_TYPE_NORM, session->scratch));
+}
+
+/* smtp_out_add_headers - output additional headers, uses session->scratch* */
+
+static int smtp_out_add_headers(SMTP_STATE *state)
+{
+    /* Prepend headers in the same order as mail_copy.c. */
+    if (smtp_cli_attr.flags & SMTP_CLI_FLAG_RETURN_PATH)
+	if (smtp_out_add_header(state, "Return-Path", "<",
+				state->request->sender, ">") < 0)
+	    return (-1);
+    if (smtp_cli_attr.flags & SMTP_CLI_FLAG_ORIG_RCPT)
+	if (smtp_out_add_header(state, "X-Original-To", "",
+			 state->request->rcpt_list.info->orig_addr, "") < 0)
+	    return (-1);
+    if (smtp_cli_attr.flags & SMTP_CLI_FLAG_DELIVERED_TO)
+	if (smtp_out_add_header(state, "Delivered-To", "",
+			   state->request->rcpt_list.info->address, "") < 0)
+	    return (-1);
+    return (0);
+}
+
 /* smtp_loop - exercise the SMTP protocol engine */
 
 static int smtp_loop(SMTP_STATE *state, NOCLOBBER int send_state,
@@ -1414,24 +1474,6 @@ static int smtp_loop(SMTP_STATE *state, NOCLOBBER int send_state,
     SMTP_RESP fake;
     int     fail_status;
 
-    /*
-     * Macros for readability.
-     */
-#define REWRITE_ADDRESS(dst, src) do { \
-	vstring_strcpy(dst, src); \
-	if (*(src) && smtp_generic_maps) \
-	    smtp_map11_internal(dst, smtp_generic_maps, \
-		smtp_ext_prop_mask & EXT_PROP_GENERIC); \
-    } while (0)
-
-#define QUOTE_ADDRESS(dst, src) do { \
-	if (*(src) && var_smtp_quote_821_env) { \
-	    quote_821_local(dst, src); \
-	} else { \
-	    vstring_strcpy(dst, src); \
-	} \
-    } while (0)
-
     /* Caution: changes to RETURN() also affect code outside the main loop. */
 
 #define RETURN(x) do { \
@@ -1502,7 +1544,7 @@ static int smtp_loop(SMTP_STATE *state, NOCLOBBER int send_state,
 	msg_panic("%s: bad sender state %d (receiver state %d)",
 		  myname, send_state, recv_state);
     smtp_stream_setup(session->stream, *xfer_timeouts[send_state],
-		      var_smtp_rec_deadline);
+		      var_smtp_req_deadline, 0);
     if ((except = vstream_setjmp(session->stream)) != 0) {
 	msg_warn("smtp_proto: spurious flush before read in send state %d",
 		 send_state);
@@ -1606,8 +1648,9 @@ static int smtp_loop(SMTP_STATE *state, NOCLOBBER int send_state,
 	case SMTP_STATE_MAIL:
 	    request->msg_stats.reuse_count = session->reuse_count;
 	    GETTIMEOFDAY(&request->msg_stats.conn_setup_done);
-	    REWRITE_ADDRESS(session->scratch2, request->sender);
-	    QUOTE_ADDRESS(session->scratch, vstring_str(session->scratch2));
+	    smtp_rewrite_generic_internal(session->scratch2, request->sender);
+	    smtp_quote_821_address(session->scratch,
+				   vstring_str(session->scratch2));
 	    vstring_sprintf(next_command, "MAIL FROM:<%s>",
 			    vstring_str(session->scratch));
 	    /* XXX Don't announce SIZE if we're going to MIME downgrade. */
@@ -1695,8 +1738,9 @@ static int smtp_loop(SMTP_STATE *state, NOCLOBBER int send_state,
 	     */
 	case SMTP_STATE_RCPT:
 	    rcpt = request->rcpt_list.info + send_rcpt;
-	    REWRITE_ADDRESS(session->scratch2, rcpt->address);
-	    QUOTE_ADDRESS(session->scratch, vstring_str(session->scratch2));
+	    smtp_rewrite_generic_internal(session->scratch2, rcpt->address);
+	    smtp_quote_821_address(session->scratch,
+				   vstring_str(session->scratch2));
 	    vstring_sprintf(next_command, "RCPT TO:<%s>",
 			    vstring_str(session->scratch));
 	    if (session->features & SMTP_FEATURE_DSN) {
@@ -1887,7 +1931,7 @@ static int smtp_loop(SMTP_STATE *state, NOCLOBBER int send_state,
 #define LOST_CONNECTION_INSIDE_DATA (except == SMTP_ERR_EOF)
 
 		smtp_stream_setup(session->stream, *xfer_timeouts[recv_state],
-				  var_smtp_rec_deadline);
+				  var_smtp_req_deadline, 0);
 		if (LOST_CONNECTION_INSIDE_DATA) {
 		    if (vstream_setjmp(session->stream) != 0)
 			RETURN(smtp_stream_except(state, SMTP_ERR_EOF,
@@ -2222,7 +2266,7 @@ static int smtp_loop(SMTP_STATE *state, NOCLOBBER int send_state,
 	if (send_state == SMTP_STATE_DOT && nrcpt > 0) {
 
 	    smtp_stream_setup(session->stream, var_smtp_data1_tmout,
-			      var_smtp_rec_deadline);
+			      var_smtp_req_deadline, var_smtp_min_data_rate);
 
 	    if ((except = vstream_setjmp(session->stream)) == 0) {
 
@@ -2257,24 +2301,16 @@ static int smtp_loop(SMTP_STATE *state, NOCLOBBER int send_state,
 							   (void *) state);
 		state->space_left = var_smtp_line_limit;
 
+		if ((smtp_cli_attr.flags & SMTP_CLI_MASK_ADD_HEADERS) != 0
+		    && smtp_out_add_headers(state) < 0)
+		    RETURN(0);
+
 		while ((rec_type = rec_get(state->src, session->scratch, 0)) > 0) {
 		    if (rec_type != REC_TYPE_NORM && rec_type != REC_TYPE_CONT)
 			break;
-		    if (session->mime_state == 0) {
-			smtp_text_out((void *) state, rec_type,
-				      vstring_str(session->scratch),
-				      VSTRING_LEN(session->scratch),
-				      (off_t) 0);
-		    } else {
-			mime_errs =
-			    mime_state_update(session->mime_state, rec_type,
-					      vstring_str(session->scratch),
-					      VSTRING_LEN(session->scratch));
-			if (mime_errs) {
-			    smtp_mime_fail(state, mime_errs);
-			    RETURN(0);
-			}
-		    }
+		    if (smtp_out_raw_or_mime(state, rec_type,
+					     session->scratch) < 0)
+			RETURN(0);
 		    prev_type = rec_type;
 		}
 
