@@ -69,8 +69,10 @@
 /*
 /*	void	tls_param_init()
 /*
-/*	int	tls_protocol_mask(plist)
+/*	int	tls_proto_mask_lims(plist, floor, ceiling)
 /*	const char *plist;
+/*	int	*floor;
+/*	int	*ceiling;
 /*
 /*	int	tls_cipher_grade(name)
 /*	const char *name;
@@ -157,10 +159,12 @@
 /*	entering a chroot jail. The "role" parameter must be TLS_ROLE_CLIENT
 /*	for clients and TLS_ROLE_SERVER for servers. Any errors are fatal.
 /*
-/*	tls_protocol_mask() returns a bitmask of excluded protocols, given
-/*	a list (plist) of protocols to include or (preceded by a '!') exclude.
-/*	If "plist" contains invalid protocol names, TLS_PROTOCOL_INVALID is
-/*	returned and no warning is logged.
+/*	tls_proto_mask_lims() returns a bitmask of excluded protocols, and
+/*	and the protocol version floor/ceiling, given a list (plist) of
+/*	protocols to include or (preceded by a '!') exclude, or constraints
+/*	of the form '>=name', '<=name', '>=hexvalue', '<=hexvalue'. If "plist"
+/*	contains invalid protocol names, TLS_PROTOCOL_INVALID is returned and
+/*	no warning is logged.
 /*
 /*	tls_cipher_grade() converts a case-insensitive cipher grade
 /*	name (high, medium, low, export, null) to the corresponding
@@ -239,8 +243,6 @@
 #include <ctype.h>
 #include <string.h>
 
-#ifdef USE_TLS
-
 /* Utility library. */
 
 #include <vstream.h>
@@ -292,13 +294,11 @@ char   *var_tls_tkt_cipher;
 char   *var_openssl_path;
 char   *var_tls_server_sni_maps;
 bool    var_tls_fast_shutdown;
-
-static MAPS *tls_server_sni_maps;
-
-#ifdef VAR_TLS_PREEMPT_CLIST
 bool    var_tls_preempt_clist;
 
-#endif
+#ifdef USE_TLS
+
+static MAPS *tls_server_sni_maps;
 
  /*
   * Index to attach TLScontext pointers to SSL objects, so that they can be
@@ -319,6 +319,19 @@ static const NAME_CODE protocol_table[] = {
     0, TLS_PROTOCOL_INVALID,
 };
 
+/*
+ * Protocol name => numeric version, for MinProtocol and MaxProtocol
+ */
+static const NAME_CODE tls_version_table[] = {
+    "None", 0,
+    SSL_TXT_SSLV3, SSL3_VERSION,
+    SSL_TXT_TLSV1, TLS1_VERSION,
+    SSL_TXT_TLSV1_1, TLS1_1_VERSION,
+    SSL_TXT_TLSV1_2, TLS1_2_VERSION,
+    TLS_PROTOCOL_TXT_TLSV1_3, TLS1_3_VERSION,
+    0, -1,
+};
+
  /*
   * SSL_OP_MUMBLE bug work-around name <=> mask conversion.
   */
@@ -555,9 +568,32 @@ void    tls_update_app_logmask(TLS_APPL_STATE *app_ctx, int log_mask)
     app_ctx->log_mask = log_mask;
 }
 
-/* tls_protocol_mask - Bitmask of protocols to exclude */
+/* parse_version - parse TLS protocol version name or hex number */
 
-int     tls_protocol_mask(const char *plist)
+static int parse_tls_version(const char *tok, int *version)
+{
+    int     code = name_code(tls_version_table, NAME_CODE_FLAG_NONE, tok);
+    char   *_end;
+    unsigned long ulval;
+
+    if (code != -1) {
+	*version = code;
+	return (0);
+    }
+    errno = 0;
+    ulval = strtoul(tok, &_end, 16);
+    if (*_end != 0
+	|| (ulval == ULONG_MAX && errno == ERANGE)
+	|| ulval > INT_MAX)
+	return TLS_PROTOCOL_INVALID;
+
+    *version = (int) ulval;
+    return (0);
+}
+
+/* tls_proto_mask_lims - protocols to exclude and floor/ceiling */
+
+int     tls_proto_mask_lims(const char *plist, int *floor, int *ceiling)
 {
     char   *save;
     char   *tok;
@@ -571,9 +607,15 @@ int     tls_protocol_mask(const char *plist)
 	return (res); \
     } while (0)
 
+    *floor = *ceiling = 0;
+
     save = cp = mystrdup(plist);
     while ((tok = mystrtok(&cp, CHARS_COMMA_SP ":")) != 0) {
-	if (*tok == '!')
+	if (strncmp(tok, ">=", 2) == 0)
+	    code = parse_tls_version(tok + 2, floor);
+	else if (strncmp(tok, "<=", 2) == 0)
+	    code = parse_tls_version(tok + 2, ceiling);
+	else if (*tok == '!')
 	    exclude |= code =
 		name_code(protocol_table, NAME_CODE_FLAG_NONE, ++tok);
 	else
@@ -834,8 +876,11 @@ void    tls_get_signature_params(TLS_SESS_STATE *TLScontext)
     int     nid;
     SSL    *ssl = TLScontext->con;
     int     srvr = SSL_is_server(ssl);
-    X509   *cert;
-    EVP_PKEY *pkey = 0;
+    EVP_PKEY *dh_pkey = 0;
+    X509   *local_cert;
+    EVP_PKEY *local_pkey = 0;
+    X509   *peer_cert;
+    EVP_PKEY *peer_pkey = 0;
 
 #ifndef OPENSSL_NO_EC
     EC_KEY *eckey;
@@ -847,21 +892,21 @@ void    tls_get_signature_params(TLS_SESS_STATE *TLScontext)
     if (SSL_version(ssl) < TLS1_3_VERSION)
 	return;
 
-    if (tls_get_peer_dh_pubkey(ssl, &pkey)) {
-	switch (nid = EVP_PKEY_id(pkey)) {
+    if (tls_get_peer_dh_pubkey(ssl, &dh_pkey)) {
+	switch (nid = EVP_PKEY_id(dh_pkey)) {
 	default:
 	    kex_name = OBJ_nid2sn(EVP_PKEY_type(nid));
 	    break;
 
 	case EVP_PKEY_DH:
 	    kex_name = "DHE";
-	    TLScontext->kex_bits = EVP_PKEY_bits(pkey);
+	    TLScontext->kex_bits = EVP_PKEY_bits(dh_pkey);
 	    break;
 
 #ifndef OPENSSL_NO_EC
 	case EVP_PKEY_EC:
 	    kex_name = "ECDHE";
-	    eckey = EVP_PKEY_get0_EC_KEY(pkey);
+	    eckey = EVP_PKEY_get0_EC_KEY(dh_pkey);
 	    nid = EC_GROUP_get_curve_name(EC_KEY_get0_group(eckey));
 	    kex_curve = EC_curve_nid2nist(nid);
 	    if (!kex_curve)
@@ -869,7 +914,7 @@ void    tls_get_signature_params(TLS_SESS_STATE *TLScontext)
 	    break;
 #endif
 	}
-	EVP_PKEY_free(pkey);
+	EVP_PKEY_free(dh_pkey);
     }
 
     /*
@@ -878,20 +923,20 @@ void    tls_get_signature_params(TLS_SESS_STATE *TLScontext)
      * data on clients requires at least 1.1.1a.
      */
     if (srvr || SSL_get_signature_nid(ssl, &nid))
-	cert = SSL_get_certificate(ssl);
+	local_cert = SSL_get_certificate(ssl);
     else
-	cert = 0;
+	local_cert = 0;
 
     /* Signature algorithms for the local end of the connection */
-    if (cert) {
-	pkey = X509_get0_pubkey(cert);
+    if (local_cert) {
+	local_pkey = X509_get0_pubkey(local_cert);
 
 	/*
 	 * Override the built-in name for the "ECDSA" algorithms OID, with
 	 * the more familiar name.  For "RSA" keys report "RSA-PSS", which
 	 * must be used with TLS 1.3.
 	 */
-	if ((nid = EVP_PKEY_type(EVP_PKEY_id(pkey))) != NID_undef) {
+	if ((nid = EVP_PKEY_type(EVP_PKEY_id(local_pkey))) != NID_undef) {
 	    switch (nid) {
 	    default:
 		locl_sig_name = OBJ_nid2sn(nid);
@@ -900,13 +945,13 @@ void    tls_get_signature_params(TLS_SESS_STATE *TLScontext)
 	    case EVP_PKEY_RSA:
 		/* For RSA, TLS 1.3 mandates PSS signatures */
 		locl_sig_name = "RSA-PSS";
-		SIG_PROP(TLScontext, srvr, bits) = EVP_PKEY_bits(pkey);
+		SIG_PROP(TLScontext, srvr, bits) = EVP_PKEY_bits(local_pkey);
 		break;
 
 #ifndef OPENSSL_NO_EC
 	    case EVP_PKEY_EC:
 		locl_sig_name = "ECDSA";
-		eckey = EVP_PKEY_get0_EC_KEY(pkey);
+		eckey = EVP_PKEY_get0_EC_KEY(local_pkey);
 		nid = EC_GROUP_get_curve_name(EC_KEY_get0_group(eckey));
 		locl_sig_curve = EC_curve_nid2nist(nid);
 		if (!locl_sig_curve)
@@ -914,6 +959,7 @@ void    tls_get_signature_params(TLS_SESS_STATE *TLScontext)
 		break;
 #endif
 	    }
+	    /* No X509_free(local_cert) */
 	}
 
 	/*
@@ -924,15 +970,15 @@ void    tls_get_signature_params(TLS_SESS_STATE *TLScontext)
 	    locl_sig_dgst = OBJ_nid2sn(nid);
     }
     /* Signature algorithms for the peer end of the connection */
-    if ((cert = SSL_get_peer_certificate(ssl)) != 0) {
-	pkey = X509_get0_pubkey(cert);
+    if ((peer_cert = SSL_get_peer_certificate(ssl)) != 0) {
+	peer_pkey = X509_get0_pubkey(peer_cert);
 
 	/*
 	 * Override the built-in name for the "ECDSA" algorithms OID, with
 	 * the more familiar name.  For "RSA" keys report "RSA-PSS", which
 	 * must be used with TLS 1.3.
 	 */
-	if ((nid = EVP_PKEY_type(EVP_PKEY_id(pkey))) != NID_undef) {
+	if ((nid = EVP_PKEY_type(EVP_PKEY_id(peer_pkey))) != NID_undef) {
 	    switch (nid) {
 	    default:
 		peer_sig_name = OBJ_nid2sn(nid);
@@ -941,13 +987,13 @@ void    tls_get_signature_params(TLS_SESS_STATE *TLScontext)
 	    case EVP_PKEY_RSA:
 		/* For RSA, TLS 1.3 mandates PSS signatures */
 		peer_sig_name = "RSA-PSS";
-		SIG_PROP(TLScontext, !srvr, bits) = EVP_PKEY_bits(pkey);
+		SIG_PROP(TLScontext, !srvr, bits) = EVP_PKEY_bits(peer_pkey);
 		break;
 
 #ifndef OPENSSL_NO_EC
 	    case EVP_PKEY_EC:
 		peer_sig_name = "ECDSA";
-		eckey = EVP_PKEY_get0_EC_KEY(pkey);
+		eckey = EVP_PKEY_get0_EC_KEY(peer_pkey);
 		nid = EC_GROUP_get_curve_name(EC_KEY_get0_group(eckey));
 		peer_sig_curve = EC_curve_nid2nist(nid);
 		if (!peer_sig_curve)
@@ -963,6 +1009,8 @@ void    tls_get_signature_params(TLS_SESS_STATE *TLScontext)
 	 */
 	if (SSL_get_peer_signature_nid(ssl, &nid) && nid != NID_undef)
 	    peer_sig_dgst = OBJ_nid2sn(nid);
+
+	X509_free(peer_cert);
     }
     if (kex_name) {
 	TLScontext->kex_name = mystrdup(kex_name);
